@@ -815,3 +815,17 @@ elasticsearch_allow_reallocate() {
 # Initialize installation - run basic checks and detect OS info
 set_os_info
 std_checks
+
+
+update_java_logging_configs() {
+	cp /usr/local/nagioslogserver/logstash/config/jvm.options /usr/local/nagioslogserver/logstash/config/jvm.options.bak
+	cp subcomponents/logstash/mods/jvm.options /usr/local/nagioslogserver/logstash/config/jvm.options
+	cp /usr/local/nagioslogserver/logstash/config/log4j2.properties /usr/local/nagioslogserver/logstash/config/log4j2.properties.bak
+	cp subcomponents/logstash/mods/log4j2.properties /usr/local/nagioslogserver/logstash/config/log4j2.properties
+	cp /usr/local/nagioslogserver/opensearch/config/jvm.options /usr/local/nagioslogserver/opensearch/config/jvm.options.bak
+	cp subcomponents/opensearch/mods/opensearch/jvm.options /usr/local/nagioslogserver/opensearch/config/jvm.options
+	cp /usr/local/nagioslogserver/opensearch/config/log4j2.properties /usr/local/nagioslogserver/opensearch/config/log4j2.properties.bak
+	cp subcomponents/opensearch/mods/opensearch/log4j2.properties /usr/local/nagioslogserver/opensearch/config/log4j2.properties
+	sed -i s/rootLogger.level\ =\ info/rootLogger.level\ =\ warn/1 /usr/local/nagioslogserver/opensearch/config/log4j2.properties
+	sed -i s/^#\ log.level:\ info$/log.level:\ warn/1 /usr/local/nagioslogserver/logstash/config/logstash.yml
+}
\ No newline at end of file
