@@ -26,7 +26,10 @@ $config['hosts_file'] = $config['backend_dir'].'/var/cluster_hosts';
 
 // Backend constants
 $node_id = trim(file_get_contents($config['node_uuid_file']));
-$cluster_id = trim(file_get_contents($config['cluster_uuid_file']));
+$cluster_id = 'nagios_opensearch';
+if (file_exists($config['cluster_uuid_file'])) {
+    $cluster_id = trim(file_get_contents($config['cluster_uuid_file']));
+}
 define("NODE", $node_id);
 define("CLUSTER", $cluster_id);
 
