@@ -206,7 +206,7 @@ function do_maintenance_trim_alert_index($trim_time)
     return $msg;
 }
 
-function get_open_logstash_indices_older_than($index_time) {
+function get_logstash_indices_older_than($index_time, $include_closed=false) {
     $ci =& get_instance();
     $opensearch_password = $ci->config->item('opensearch_password');
 
@@ -227,7 +227,7 @@ function get_open_logstash_indices_older_than($index_time) {
     $index_older_than_date = date('Y.m.d', strtotime("-$index_time days"));
     $older_indices = array();
     foreach ($indices as $index) {
-        if ($index['status'] == 'open' &&
+        if (($index['status'] == 'open' || $include_closed === true) &&
             preg_match('/^logstash-\d\d\d\d\.\d\d\.\d\d/', $index['index']) ) {
             if ($index['index'] < "logstash-$index_older_than_date") {
                 $older_indices[] = $index['index'];
@@ -243,7 +243,7 @@ function do_maintenance_close_indices($close_time) {
     $ci =& get_instance();
     $opensearch_password = $ci->config->item('opensearch_password');
 
-    $close_indices = get_open_logstash_indices_older_than($close_time);
+    $close_indices = get_logstash_indices_older_than($close_time);
     $session = curl_init();
 
     $close_results = '';
@@ -275,7 +275,7 @@ function do_maintenance_delete_indices($delete_time) {
     $ci =& get_instance();
     $opensearch_password = $ci->config->item('opensearch_password');
 
-    $delete_indices = get_open_logstash_indices_older_than($delete_time);
+    $delete_indices = get_logstash_indices_older_than($delete_time, true);
     $session = curl_init();
 
     syslog(LOG_INFO, "Deleting Got list of indices: " . implode(',', $delete_indices));
@@ -314,7 +314,7 @@ function do_maintenance_optimize($optimize_time)
     $ci =& get_instance();
     $opensearch_password = $ci->config->item('opensearch_password');
 
-    $optimize_indices = get_open_logstash_indices_older_than($optimize_time);
+    $optimize_indices = get_logstash_indices_older_than($optimize_time);
 
     $optimization_results = '';
     $session = curl_init();
@@ -638,6 +638,65 @@ function change_timezone($args=array())
     return true;
 }
 
+function change_log_location($args=array()) {
+    $ci =& get_instance();
+
+    $backend_dir = $ci->config->item('backend_dir');
+
+    $ymlfiles = ['opensearch/config/opensearch.yml', 'logstash/config/logstash.yml'];
+
+    $log_location = $args['log_location'];
+    sleep($args['wait_time']);
+
+    foreach ($ymlfiles as $ymlfile) {
+        $file_name = "$backend_dir/$ymlfile";
+        $yaml = file_get_contents($file_name);
+
+        if ($ymlfile == 'opensearch/config/opensearch.yml') {
+            if ($log_location == 'default') {
+                $log_line = 'path.logs: /usr/local/nagioslogserver/opensearch/logs';
+            } else {
+                $log_line = 'path.logs: /var/log/opensearch';
+            }
+            $service = 'opensearch';
+        } else {
+            if ($log_location == 'default') {
+                $log_line = 'path.logs: /usr/local/nagioslogserver/logstash/logs';
+            } else {
+                $log_line = 'path.logs: /var/log/logstash';
+            }
+            $service = 'logstash';
+        }
+        if ($yaml === false) {
+            return;
+        }
+
+        $yml_contents = explode("\n", $yaml);
+
+        $new_yaml = [];
+
+        $did_update_yaml = false;
+
+        foreach ($yml_contents as $line) {
+            if (preg_match('/^path.logs/', $line)) {
+                $did_update_yaml = true;
+                $line = $log_line;
+            }
+            $new_yaml[] = $line;
+        }
+
+        if (!$did_update_yaml) {
+            $new_yaml[] = $log_line;
+        }
+
+        file_put_contents($file_name, implode("\n", $new_yaml));
+
+        $cmd = "sudo systemctl restart $service";
+        exec($cmd, $o, $r);
+    }
+
+    return true;
+}
 
 function start_logdata_migration($args=array()) {
     syslog(LOG_ERR, "Migration start_logdata_migration args: " . json_encode($args));
@@ -714,7 +773,6 @@ function reconfigure_opensearch($args=array()) {
 
 function apply_instance_roles($args=array()) {
     $ci =& get_instance();
-    $scripts_dir = $ci->config->item('scripts_dir');
     $backend_dir = $ci->config->item('backend_dir');
 
     $es = new Elasticsearch(array('index' => 'nagioslogserver'));
@@ -724,7 +782,7 @@ function apply_instance_roles($args=array()) {
         return false;
     }
 
-    $abcd = json_decode($global['_source']['requested_roles'], true);
+    $requested_roles = json_decode($global['_source']['requested_roles'], true);
 
     $ymlfiles = ['opensearch.yml'];
 
@@ -758,10 +816,12 @@ function apply_instance_roles($args=array()) {
 
         $new_roles = [];
 
-        foreach ($abcd['roles'] as $address => $node) {
+        foreach ($requested_roles['roles'] as $address => $node) {
             if ($node['name'] == $node_name) {
                 foreach ($node as $role => $value) {
-                    if ($value === true) {
+                    // regardless of the vaue every node has the data role, at least until
+                    // the exclusion list has moved the shards off of the node.
+                    if ($value === true || $role == 'data') {
                         $new_roles[] = $role;
                     }
                 }
