@@ -109,7 +109,7 @@ function create_auth_connection()
 //  API Helper Functions
 // ====================================
 
-function grab_ad_folders($folder="", $type="container")
+function grab_ad_folders($folder="", $type="container", $search="", $target_page=1, $page_size=50)
 {
     $conn = create_auth_connection();
     $output = false;
@@ -117,28 +117,42 @@ function grab_ad_folders($folder="", $type="container")
 
     if ($type == "organizationalUnit") {
         if ($conn->type == "ad") {
-            $list_array = $conn->folder()->listing($folder, AdLDAP::ADLDAP_FOLDER, false);
+            $list_array = $conn->folder()->listing($folder, AdLDAP::ADLDAP_FOLDER, false, null, $search, $target_page, $page_size);
         } else {
-            $list_array = $conn->folder_listing($folder, BasicLDAP::LDAP_FOLDER);
+            $list_array = $conn->folder_listing($folder, BasicLDAP::LDAP_FOLDER, $search, $target_page, $page_size);
         }
-        $output = check_validity($list_array);
+        $output = array(
+            'entries' => check_validity($list_array['entries']),
+            'target_page' => $target_page,
+            'page_count' => $list_array['pageCount'],
+            'page_size' => $list_array['pageSize']
+        );
     } else if ($type == "container" || $type == "nsContainer") {
         if ($conn->type == "ad") {
-            $list_array = $conn->folder()->listing($folder, $conn->getContainerRdnAttr(), false);
+            $list_array = $conn->folder()->listing($folder, adLDAP::ADLDAP_FOLDER, false, null, $search, $target_page, $page_size);
         } else {
-            $list_array = $conn->folder_listing($folder, $conn->getContainerRdnAttr());
+            $list_array = $conn->folder_listing($folder, basicLDAP::LDAP_FOLDER, $search, $target_page, $page_size);
         }
-        $output = check_validity($list_array);
+        $output = array(
+            'entries' => check_validity($list_array['entries']),
+            'target_page' => $target_page,
+            'page_count' => $list_array['pageCount'],
+            'page_size' => $list_array['pageSize']
+        );
     } else if ($type == "group") {
         $folder = grab_array_var($folder, "0");
-        $ad_array = $conn->group()->members($folder);
-        $output = $ad_array;
+        $ad_array = $conn->group()->members($folder, null, $search, $target_page, $page_size);
+        $output = array(
+            'entries' => $ad_array,
+            'target_page' => $target_page,
+            'page_count' => 1,
+            'page_size' => 50
+        );
     }
 
     if ($output) {
         $output['base_dn'] = $base_dn;
     }
-
     return $output;
 }
 
@@ -153,17 +167,14 @@ function display_nav_window($array_to_enum, $new_list=0)
     $dont_show = array("System", "Program Data", "ForeignSecurityPrincipals", "Managed Service Accounts");
     $output_html = '';
     if ($new_list) {
-        // echo '<ul class="ad-list folder-list">';
         $output_html .= '<ul class="ad-list folder-list">';
     } else {
-        // echo '<ul class="ad-list sub-list">';
         $output_html .= '<ul class="ad-list sub-list">';
     }
+    $base_ou = grab_base_ou($array_to_enum['entries']);
 
-    $base_ou = grab_base_ou($array_to_enum);
-
-    if (!($array_to_enum == false)) {
-        foreach ($array_to_enum as $obj) {
+    if (!($array_to_enum['entries'] == false)) {
+        foreach ($array_to_enum['entries'] as $obj) {
             if (is_array($obj)) {
                 $path = grab_path($obj);
                 if ($base_ou) {
@@ -200,47 +211,62 @@ function display_nav_window($array_to_enum, $new_list=0)
     echo json_encode(Array($output_html, $base_ou));
 }
 
-function display_users($array_to_enum, $location, $type, $selected)
+function display_users($array_to_enum, $location, $type, $selected, $search = "")
 {
     // List of usernames not to show...
     $dont_show = array("krbtgt");
-
     if ($location) {
        $location = grab_array_var($location, "0");
     }
     $conn = create_auth_connection();
+    $base_dn = $conn->getBaseDn();
     $person_exists = false;
     $all_checked = true;
     $printed = false;
 
-    if (!empty($array_to_enum)) {
-        echo '<ul class="ad-list user-list">';
+    $current_page = $array_to_enum['target_page'];
+    $page_count = $array_to_enum['page_count'];
+    $page_size = $array_to_enum['page_size'];
+
+    $user_png = base_url("media/icons/user.png");
+    $add_user_text = _('Add user');
+    $show_full_dn_text = _("Show full DN (distinguished name)");
+
+    if (!empty($array_to_enum['entries'])) {
         if ($type == "group") {
-            foreach ($array_to_enum as $username)
+            foreach ($array_to_enum['entries'] as $username)
             {
                 $person_exists = true;
                 $userinfo = $conn->user()->info($username, array("displayname"));
                 $displayname = $userinfo[0]["displayname"][0];
                 $dn = $userinfo[0]["dn"];
+                $full_display_name = ($username == $displayname) ? $username : $displayname . ' (' . $username . ')';
+                $checkbox_value = ($conn->type == "ad")  ? $username : $dn;
+                $is_checkbox_checked = '';
+                if (in_array($dn, $selected)) {
+                     $is_checkbox_checked = "checked";
+                } else {
+                    $all_checked = false;
+                }
 
-                ?>
+                $users_html .= '
                 <li class="tw-px-8">
                     <div class="tw-flex tw-items-center tw-space-x-2">
-                        <input type="checkbox" class="ad-checkbox ui-checkbox-group-item" value="<?php if ($conn->type == "ad") { echo $username; } else { echo $dn; } ?>" <?php if (in_array($username, $selected)) { echo "checked"; } else { $all_checked = false; } ?>>
+                        <input type="checkbox" class="ad-checkbox ui-checkbox-group-item" value="'.$checkbox_value.'"'.$is_checkbox_checked.'>
                         <label class="checkbox tw-flex" for="ad-checkbox">
-                            <img class="table-icon" src="<?php echo base_url("media/icons/user.png"); ?>" border="0" alt="<?php echo _('Add user'); ?>" title="<?php echo _('Add user'); ?>" style="">
-                            <?php echo $displayname; ?> (<?php echo $username; ?>)
+                            <img class="table-icon" src="'.$user_png.'" border="0" alt="'.$add_user_text.'" title="'.$add_user_text.'" style="">
+                            '.$full_display_name.'
                         </label>
                     </div>
-                    <div class="tw-flex tw-items-center tw-gap-2 tw-h-6 tw-pl-6">
-                        <i class="fa fa-plus-square-o user-toggle-show-dn" title="<?php echo _("Show full DN (distinguished name)"); ?>"></i>
-                        <div class="user-dn hide">DN: <?php echo $dn; ?></div>
+                    <div class="tw-flex tw-items-center tw-gap-2 tw-pl-1">
+                        <i class="fa fa-plus-square-o user-toggle-show-dn" title="'.$show_full_dn_text.'"></i>
+                        <div class="user-dn hide">DN: '.$dn.'</div>
                     </div>
                 </li>
-                <?php
+                ';
             }
         } else {
-            foreach ($array_to_enum as $obj)
+            foreach ($array_to_enum['entries'] as $obj)
             {
                 if (is_array($obj)) {
 
@@ -249,53 +275,74 @@ function display_users($array_to_enum, $location, $type, $selected)
                     if (in_array(strtolower($type), $valid_units)) {
                         $username = grab_user_name($type, $obj);
                         $dn = grab_full_dn($obj);
+                        $displayname = grab_dn($obj);
 
                         if (in_array($username, $dont_show)) {
                             continue;
                         }
-
+                        $username = (isset($username)) ? $username : $displayname;
+                        $full_display_name = ($username == grab_dn($obj)) ? $username : $displayname . ' (' . $username . ')';
                         $image = grab_image_for_type($obj);
                         $person_exists = true;
-                        ?>
+                        $checkbox_value = ($conn->type == "ad") ? $username : $dn;
+                        $is_checkbox_disabled = (is_computer($obj)) ? 'disabled' : '';
+                        $is_checkbox_checked = '';
+                        if (in_array($dn, $selected)) {
+                            $is_checkbox_checked = 'checked';
+                        } else {
+                             $all_checked = false;
+                        }
+                        $table_icon_image = base_url('media/icons/'.$image);
+
+                        $users_html .= '
                         <li class="tw-px-8">
-                        <div class="tw-flex tw-items-center tw-space-x-2">
-                            <input type="checkbox" class="ad-checkbox ui-checkbox-group-item" value="<?php if ($conn->type == "ad") { echo $username; } else { echo $dn; } ?>" <?php if (is_computer($obj)) { echo "disabled"; } ?> <?php if (in_array($username, $selected)) { echo "checked"; } else { $all_checked = false; } ?>>
-                            <label class="checkbox tw-flex">
-                                <img class="table-icon" src="<?php echo base_url('media/icons/'.$image); ?>" border="0" alt="<?php echo _("Add user"); ?>" title="<?php echo _("Add user"); ?>" style="">
-                                <?php echo grab_dn($obj); ?> (<?php echo $username; ?>)
-                            </label>
-                        </div>
-                            <div class="tw-flex tw-items-center tw-gap-2 tw-h-6 tw-pl-6">
-                                <i class="fa fa-plus-square-o user-toggle-show-dn" title="<?php echo _("Show full DN (distinguished name)"); ?>"></i>
-                                <div class="user-dn hide">DN: <?php echo $dn; ?></div>
+                            <div class="tw-flex tw-flex-row">
+                                <div class="tw-flex tw-items-center tw-space-x-2">
+                                    <input type="checkbox" class="ad-checkbox ui-checkbox-group-item" data-displayname="'.$displayname.'" value="'.$checkbox_value.'"'.$is_checkbox_disabled.' '.$is_checkbox_checked.'>                                        
+                                    <label class="checkbox tw-flex">
+                                        <img class="table-icon" src="'.$table_icon_image.'" border="0" alt="'.$add_user_text.'" title="'.$add_user_text.'">
+                                        '.$full_display_name.'
+                                    </label>
+                                </div>
+                                <div class="tw-flex tw-items-center tw-gap-2 tw-pl-1">
+                                    <i class="fa fa-plus-square-o user-toggle-show-dn" title="'.$show_full_dn_text.'"></i>
+                                </div>
                             </div>
+                            <div class="user-dn hide">DN: '.$dn.'</div>
                         </li>
-                        <?php
+                        ';
                     }
                 }
             }
         }
-        if ($person_exists) {
-        ?>
-            <li class="tw-mt-2.5 tw-px-8">
-                <label class="checkbox">
-                    <input type="checkbox" class="toggle-users ui-checkbox-group-item" <?php if ($all_checked) { echo "checked"; } ?>>
-                    <span><?php if ($all_checked) { echo _("Select None"); } else { echo _("Select All"); } ?></span>
-                </label>
-            </li>
-        <?php
+    }
+    if ($person_exists) {
+        $page_controls = get_page_controls($all_checked, $current_page, $page_count, $page_size, $search);
+        $users_html = '<ul class="ad-list user-list">' . $page_controls . $users_html;
+    } else {
+        $page_controls = get_page_controls(false, $current_page, $page_count, $page_size, $search);
+        if ($search != "") {
+            $no_results_text = _("No users or computers found matching your search.");
+            $search_tips = '<p class="tw-font-bold tw-p-2">'._("Search Tips:").'</p>
+                <ul class="tw-p-2">
+                    <li class="tw-list-disc tw-ml-4">'._("Try using the wildcard character (*)").'</li>
+                    <li class="tw-list-disc tw-ml-4">'._("Search for a user's name, rather than their username").'</li>
+                    <li class="tw-list-disc tw-ml-4">'._("Try different or fewer keywords").'</li>
+                    <li class="tw-list-disc tw-ml-4">'._("Check your spelling").'</li>
+                </ul>
+            </li>';
         } else {
-            echo '<li>'._("No users or computers found in this object.").'</li>';
-            $printed = true;
+            $no_results_text = _("No users or computers found in this object.");
         }
-        echo '<ul>';
-    }
-
-    if (!$person_exists && !$printed) {
-        echo '<ul class="ad-list user-list">';
-        echo '<li class="tw-p-2">'._("No users or computers found in this object.").'</li>';
-        echo "</ul>";
+        $users_html .= '
+        <ul class="ad-list user-list">
+            ' . $page_controls . '
+            <li class="tw-p-2">'.$no_results_text.'</li>
+            '.$search_tips.'
+        </ul>
+        ';
     }
+    echo $users_html;
 }
 
 // ==============================
@@ -426,3 +473,77 @@ function grab_base_ou($enum) {
 
     return (count($base_ou)) ? $base_ou : false;
 }
+
+function get_page_controls($all_checked, $current_page = 1, $page_count = 1, $page_size = 50, $search = "") {
+
+    
+    $back_btn_disabled = ($current_page == 1) ? 'disabled' : '';
+    $forward_btn_disabled = ($current_page == $page_count) ? 'disabled' : '';
+    
+    $page_sizes = [50, 100, 250, 500, 1000];
+    $page_size_options = '';
+    foreach ($page_sizes as $option) {
+        $selected = ($page_size == $option) ? 'selected' : '';
+        $page_size_options .= '<option value="' . $option . '" ' . $selected . '>' . $option . ' ' . _('Per Page') . '</option>';
+    }
+
+    $close_button = '';
+    if (!empty($search)) {
+        $close_button = '
+        <button id="ad-search-clear-btn" class="ui-btn ui-btn-outlined">
+            <span class="material-symbols-outlined md-pointer md-20 md-400">close</span>
+        </button>';
+    }
+
+    $all_checked_state = ($all_checked) ? 'checked' : '';
+    $check_all_text = ($all_checked) ? _("Select None") : _("Select All");
+
+    $html_page_controls = '
+        <div class="form-inline tw-flex tw-flex-row tw-justify-between tw-m-5">
+            <div class="tw-flex">
+                <div class="tw-w-[200px]">
+                    <input type="text" id="ad-search" class="ui-input" value="' . $search . '" placeholder="Search this folder...">
+                </div>
+                <button id="ad-search-btn" class="ui-btn ui-btn-outlined tw-ml-1" ><span class="material-symbols-outlined md-pointer md-20 md-400">search</span></button>
+                ' . $close_button . '
+            </div>
+            <div class="ad-page-control-container tw-flex tw-justify-between  tw-w-72">
+                <button class="ui-btn ui-btn-outlined ad-page-btn" ' . $back_btn_disabled . ' value="' . ($current_page - 1) . '" title="' .  _('Previous Page') . '">
+                    <span class="material-symbols-outlined md-400 md-18">
+                        keyboard_arrow_left
+                    </span>
+                </button>
+                <span class="ad-pager tw-w-48 tw-flex tw-flex-row tw-border tw-items-center tw-justify-evenly tw-rounded-md tw-h-9 tw-px-2">
+                    <span class="tw-pr-2">
+                        ' . _('Page') . '
+                    </span>
+                    <span class="tw-flex tw-items-center">
+                        <span class="tw-block tw-w-12">
+                            <input type="text" id="ad-pager-page-number" class="ui-input tw-p-1" name="page" value="' . $current_page . '">
+                        </span>
+                        <span class="tw-px-2">/</span>
+                        <span class="pagination-total">' . $page_count . '</span>
+                    </span>
+                </span>
+                <button class="ui-btn ui-btn-outlined ad-page-btn" ' . $forward_btn_disabled . ' value="' . ($current_page + 1) . '" title="' .  _('Next Page') . '">
+                    <span class="material-symbols-outlined md-400 md-18">
+                        keyboard_arrow_right
+                    </span>
+                </button>
+            </div>
+            <div>
+                <select name="ad-num-records" class="ui-select num-records">
+                    ' . $page_size_options . '
+                </select>
+                <input type="hidden" id="current-page-size" value="'.$page_size.'"/>
+            </div>
+            <div class="tw-border tw-flex tw-items-center tw-rounded-md tw-mr-2 tw-p-1.5">
+                <label class="checkbox">
+                    <input type="checkbox" class="toggle-users ui-checkbox-group-item " '.$all_checked_state.'>
+                    <span>'.$check_all_text.'</span>
+                </label>
+            </div>
+        </div>';
+
+    return $html_page_controls;
+}
