@@ -13,6 +13,22 @@ function get_user_theme()
     return $user['theme'];
 }
 
+function get_spacings()
+{
+    $spacings = array('comfortable' => _('Comfortable'), 'compact' => _('Compact')); 
+    return $spacings;
+}
+
+function get_user_spacing()
+{
+    $ci =& get_instance();
+    $user = $ci->users->get_user();
+    if (empty($user['spacing'])) {
+        return 'comfortable';
+    }
+    return $user['spacing'];
+}
+
 function get_user_mode()
 {
     $ci =& get_instance();
