@@ -68,7 +68,8 @@ class AdLDAP {
     const ADLDAP_SECURITY_LOCAL_GROUP = 536870912;
     const ADLDAP_DISTRIBUTION_LOCAL_GROUP = 536870913;
     const ADLDAP_FOLDER = 'OU';
-    const ADLDAP_CONTAINER = 'CN';
+
+    protected $containerRdnAttribute = 'CN';
     
     /**
     * The default port for LDAP non-SSL connections
@@ -723,7 +724,12 @@ class AdLDAP {
                 // This should never happen in theory
                 throw new adLDAPException('Rebind to Active Directory failed. AD said: ' . $this->getLastError());
             } 
-        } 
+        }
+
+        // Detect and store the container/user RDN attribute
+        if ($ret && empty($this->containerRdnAttr)) {
+            $this->detectRdnAttribute($username);
+        }
         
         return $ret;
     }
@@ -875,6 +881,10 @@ class AdLDAP {
         if (count($mod)==0){ return (false); }
         return ($mod);
     }
+
+    public function getContainerRdnAttr() {
+        return $this->containerRdnAttr;
+    }
     
     /**
     * Convert 8bit characters e.g. accented characters to UTF8 encoded characters
@@ -934,6 +944,37 @@ class AdLDAP {
         return true;
     }
 
+    /**
+     * Detects the first RDN used for the given username and stores it.
+     */
+    protected function detectRdnAttribute($username) {
+        if (!empty($this->containerRdnAttr)) {
+            return; // already detected
+        }
+
+        // Use OR filter to find DN by CN, UID, or sAMAccountName
+        $filter = sprintf(
+            '(|(cn=%s)(uid=%s)(sAMAccountName=%s))',
+            ldap_escape($username, '', LDAP_ESCAPE_FILTER),
+            ldap_escape($username, '', LDAP_ESCAPE_FILTER),
+            ldap_escape($username, '', LDAP_ESCAPE_FILTER)
+        );
+
+        $search = @ldap_search($this->ldapConnection, $this->baseDn, $filter, ['dn']);
+        if ($search) {
+            $entries = ldap_get_entries($this->ldapConnection, $search);
+            if ($entries['count'] > 0 && !empty($entries[0]['dn'])) {
+                if (preg_match('/^([a-zA-Z]+)=/', $entries[0]['dn'], $matches)) {
+                    $this->containerRdnAttr = strtoupper($matches[1]); // CN, UID, sAMAccountName, etc.
+                }
+            }
+        }
+
+        // fallback if detection fails
+        if (empty($this->containerRdnAttr)) {
+            $this->containerRdnAttr = 'CN';
+        }
+    }
 }
 
 /**
