@@ -78,7 +78,7 @@ class adLDAPFolders {
     * @param bool $type Specify a type of object to search for
     * @return array
     */
-    public function listing($folderName = NULL, $dnType = adLDAP::ADLDAP_FOLDER, $recursive = NULL, $type = NULL, $search = "") 
+    public function listing($folderName = NULL, $dnType = adLDAP::ADLDAP_FOLDER, $recursive = NULL, $type = NULL, $search = "", $pageNumber = 1, $pageSize = 50) 
     {
         if ($recursive === NULL) { $recursive = $this->adldap->getRecursiveGroups(); } //use the default option if they haven't set it
         if (!$this->adldap->getLdapBind()) { return false; }
@@ -142,8 +142,8 @@ class adLDAPFolders {
         }
         else {
             // set paging control values for all branches
-            $pageSize = 1000;
             $cookie = '';
+            $pageCount = 1;
             $entries = array();
 
             // PHP Version 8.0+ moved pagination controls to the $controls object passed to LDAP query functions
@@ -158,15 +158,31 @@ class adLDAPFolders {
                         ]
                     ]];
 
-                    $sr = ldap_list($this->adldap->getLdapConnection(), $searchOu, $filter, ['objectclass', 'distinguishedname', 'samaccountname'], 0, 0, 0, LDAP_DEREF_NEVER, $controls);
+                    $sr = ldap_list(
+                        $this->adldap->getLdapConnection(),
+                        $searchOu,
+                        $filter,
+                        ['objectclass', 'distinguishedname', 'samaccountname'],
+                        0, 0, 0,
+                        LDAP_DEREF_NEVER,
+                        $controls
+                    );
 
                     $tmp = @ldap_get_entries($this->adldap->getLdapConnection(), $sr);
-                    if (is_array($tmp)) {
-                        $entries = array_merge($entries, $tmp);
+                    if ($pageCount == $pageNumber && is_array($tmp)) {
+                        $entries = $tmp;
                     }
 
                     // Parse response to retrieve new cookie
-                    ldap_parse_result($this->adldap->getLdapConnection(), $sr, $errcode, $matcheddn, $errmsg, $referrals, $resp_controls);
+                    ldap_parse_result(
+                        $this->adldap->getLdapConnection(),
+                        $sr,
+                        $errcode,
+                        $matcheddn,
+                        $errmsg,
+                        $referrals,
+                        $resp_controls
+                    );
 
                     if (isset($resp_controls[LDAP_CONTROL_PAGEDRESULTS]['value']['cookie'])) {
                         $cookie = $resp_controls[LDAP_CONTROL_PAGEDRESULTS]['value']['cookie'];
@@ -174,27 +190,63 @@ class adLDAPFolders {
                         $cookie = '';
                     }
 
+                    $pageCount++;
+
                 } while ($cookie !== '');
 
-                return $entries;
+                return [
+                    'entries' => $entries,
+                    'pageCount' => $pageCount - 1,
+                    'pageSize' => $pageSize,
+                ];
             } else if (version_compare(phpversion(), '8.0', '<') && (version_compare(phpversion(), '5.4', '>'))) {
                 do {
-                    ldap_control_paged_result($this->adldap->getLdapConnection(), $pageSize, true, $cookie);
-                    ldap_control_paged_result($this->adldap->getLdapConnection(), $pageSize, true, $cookie);
-                    $sr = ldap_list($this->adldap->getLdapConnection(), $searchOu, $filter, array('objectclass', 'distinguishedname', 'samaccountname'), 0, 0);
-                    $tmpentries = @ldap_get_entries($this->adldap->getLdapConnection(), $sr);
-                    if (is_array($tmpentries)) {
-                        $entries = array_merge($entries, $tmpentries);
+                    ldap_control_paged_result(
+                        $this->adldap->getLdapConnection(),
+                        $pageSize,
+                        true,
+                        $cookie
+                    );
+
+                    $sr = ldap_list(
+                        $this->adldap->getLdapConnection(),
+                        $searchOu,
+                        $filter,
+                        array('objectclass', 'distinguishedname', 'samaccountname'),
+                        0, 0
+                    );
+
+                    $tmp = @ldap_get_entries($this->adldap->getLdapConnection(), $sr);
+                    if ($pageCount == $pageNumber && is_array($tmp)) {
+                        $entries = $tmp;
                     }
                     ldap_control_paged_result_response($this->adldap->getLdapConnection(), $sr, $cookie);
+
+                    $pageCount++;
+
                 } while ($cookie !== null && $cookie != '');
 
-                return $entries;
+                return [
+                    'entries' => $entries,
+                    'pageCount' => $pageCount - 1,
+                    'pageSize' => $pageSize,
+                ];
             } else {
-                $sr = ldap_list($this->adldap->getLdapConnection(), $searchOu, $filter, array('objectclass', 'distinguishedname', 'samaccountname'), 0, 0);
+                $sr = ldap_list(
+                    $this->adldap->getLdapConnection(),
+                    $searchOu,
+                    $filter,
+                    array('objectclass', 'distinguishedname', 'samaccountname'),
+                    0, 0
+                );
+
                 $entries = @ldap_get_entries($this->adldap->getLdapConnection(), $sr);
                 if (is_array($entries)) {
-                    return $entries;
+                    return [
+                        'entries' => $entries,
+                        'pageCount' => $pageCount - 1,
+                        'pageSize' => $pageSize,
+                    ];
                 }
             }
         }
