@@ -327,7 +327,7 @@ class adLDAPGroups {
 
              // not a person, look for a group  
              if ($entries['count'] == 0 && $recursive == true) {  
-                $filter = "(&(objectCategory=group)(distinguishedName=" . $this->adldap->utilities()->ldapSlashes($groups[$i]) . "))";  
+                $filter = "(&(objectCategory=group)(distinguishedName=" . $this->adldap->utilities()->ldapSlashes($groups[$i]) . "))";
                 $fields = array("distinguishedname");  
                 $sr = ldap_search($this->adldap->getLdapConnection(), $this->adldap->getBaseDn(), $filter, $fields);  
                 $entries = ldap_get_entries($this->adldap->getLdapConnection(), $sr);  
@@ -354,28 +354,33 @@ class adLDAPGroups {
     * @param bool $recursive Recursively get group members
     * @return array
     */
-    public function members($group, $recursive = NULL)
+    public function members($group, $recursive = NULL, $search = "", $target_page = 1, $page_size = 50)
     {
         if (!$this->adldap->getLdapBind()){ return false; }
         if ($recursive === NULL){ $recursive = $this->adldap->getRecursiveGroups(); } // Use the default option if they haven't set it 
         // Search the directory for the members of a group
         $info = $this->info($group, array("member","cn"));
-        $users = $info[0]["member"];
+        $users = (isset($info[0]["member"])) ? $info[0]["member"] : '';
         if (!is_array($users)) {
             return false;   
         }
  
         $userArray = array();
 
+        // Add search to the filter
+        if (!empty($search)) {
+            $search = '(name='.str_replace(array('(', ')'), array('\\28', '\\29'), $search).')';
+        }
+
         for ($i=0; $i<$users["count"]; $i++){ 
-             $filter = "(&(objectCategory=person)(distinguishedName=" . $this->adldap->utilities()->ldapSlashes($users[$i]) . "))";
+             $filter = "(&(objectCategory=person)".$search."(distinguishedName=" . $this->adldap->utilities()->ldapSlashes($users[$i]) . "))";
              $fields = array("samaccountname", "distinguishedname", "objectClass");
              $sr = ldap_search($this->adldap->getLdapConnection(), $this->adldap->getBaseDn(), $filter, $fields);
              $entries = ldap_get_entries($this->adldap->getLdapConnection(), $sr);
 
              // not a person, look for a group  
              if ($entries['count'] == 0 && $recursive == true) {  
-                $filter = "(&(objectCategory=group)(distinguishedName=" . $this->adldap->utilities()->ldapSlashes($users[$i]) . "))";  
+                $filter = "(&(objectCategory=group)".$search."(distinguishedName=" . $this->adldap->utilities()->ldapSlashes($users[$i]) . "))";  
                 $fields = array("samaccountname");  
                 $sr = ldap_search($this->adldap->getLdapConnection(), $this->adldap->getBaseDn(), $filter, $fields);  
                 $entries = ldap_get_entries($this->adldap->getLdapConnection(), $sr);  
