@@ -460,13 +460,6 @@
                                     </div>
                                 </div>
                             </div>
-                            <!-- This div isn't referenced anywhere so it's just always hidden I guess? -->
-                            <!-- <div class="hide" id="raw-queries-box">
-                                <label for="a_name"><?php //echo _("Raw Query"); ?></label>
-                                <div>
-                                    <textarea id="a_raw_query"></textarea>
-                                </div>
-                            </div> -->
                             <div class="tw-flex tw-flex-row tw-gap-2 tw-items-center alert_option query_option">
                                 <div class="tw-w-1/2">
                                     <label class="ui-label tw-flex tw-flex-row tw-gap-2 tw-items-center tw-mb-0" for="a_ci">
@@ -952,23 +945,23 @@ $(document).ready(function() {
         }
 
         check = create_method_info(check);
-
         if (check.message) {
             $('#alert-message').html(check.message);
             $('#create-alert-message').addClass('alert-danger').show();
             return;
         }
-
         // Send the new check to the API
         var url = site_url+'api/check/create/1';
         if (CURRENT_ACTION == 'edit') {
-            if (!validate_interval_string(check.check_interval) || !validate_interval_string(check.lookback_period)) {
-                check.message = '<?php echo _("You must enter a valid Check Interval and Lookback Period (valid suffixes include: s, m, h, d), w, m, and y."); ?>';
-                return check;
-            }
-            if (!validate_threshold_string(check.warning) || !validate_threshold_string(check.critical)) {
-                check.message = '<?php echo _("You must enter a valid Threshold."); ?>';
-                return check;
+            if (check.alert_type === 'query') {
+                if (!validate_interval_string(check.check_interval) || !validate_interval_string(check.lookback_period)) {
+                    check.message = '<?php echo _("You must enter a valid Check Interval and Lookback Period (valid suffixes include: s, m, h, d), w, m, and y."); ?>';
+                    return check;
+                }
+                if (!validate_threshold_string(check.warning) || !validate_threshold_string(check.critical)) {
+                    check.message = '<?php echo _("You must enter a valid Threshold."); ?>';
+                    return check;
+                }
             }
 
             url = site_url+'api/check/update';
