@@ -10,7 +10,7 @@
         if ($node['_id'] == 'global') { ?>
             <div class="form-title tw-flex tw-items-center tw-gap-2">
                 <?php echo _("Global Config"); ?>
-                <a href="https://www.elastic.co/guide/en/logstash/2.4/configuration.html" target="_blank" class="no-primary tw-flex tw-items-center tw-gap-1 tw-w-fit tw-rounded-md tw-bg-secondary tw-text-sm tw-p-1 tw-ring-foreground hover:tw-no-underline hover:tw-text-inherit active:tw-ring-1 active:tw-outline-none focus:tw-ring-1 focus:tw-outline-none focus:tw-ring focus:tw-no-underline focus:tw-text-inherit">
+                <a href="https://www.elastic.co/docs/reference/logstash/creating-logstash-pipeline" target="_blank" class="no-primary tw-flex tw-items-center tw-gap-1 tw-w-fit tw-rounded-md tw-bg-secondary tw-text-sm tw-p-1 tw-ring-foreground hover:tw-no-underline hover:tw-text-inherit active:tw-ring-1 active:tw-outline-none focus:tw-ring-1 focus:tw-outline-none focus:tw-ring focus:tw-no-underline focus:tw-text-inherit">
                     <?php echo _("Docs"); ?>
                     <i class="material-symbols-outlined small">open_in_new</i>
                 </a>
@@ -161,10 +161,6 @@
                     <div class="tw-flex tw-justify-between tw-items-center tw-mb-4">
                         <div class="tw-flex tw-flex-row tw-gap-2 tw-items-center tw-justify-center">
                             <h4 class="tw-text-foreground tw-font-medium"><?php echo _("Inputs"); ?></h4>
-                            <a href="https://assets.nagios.com/downloads/nagios-log-server/docs/Listening-On-Privileged-Ports-With-Nagios-Log-Server.pdf" target="_blank" class="no-primary tw-flex tw-items-center tw-gap-1 tw-w-fit tw-rounded-md tw-bg-secondary tw-text-sm tw-p-1 tw-ring-foreground hover:tw-no-underline hover:tw-text-inherit active:tw-ring-1 active:tw-outline-none focus:tw-ring-1 focus:tw-outline-none focus:tw-ring focus:tw-no-underline focus:tw-text-inherit">
-                                <?php echo _("Privileged Ports");?>
-                                <i class="material-symbols-outlined small">open_in_new</i>
-                            </a>
                         </div>
                         <?php if (user_has_permission('configure', array('all'))) { ?>
                         <div>
